@@ -1,17 +1,55 @@
-import json
+import configparser
 import uuid
 
+from typing import Text
+
 from .coinbase_auth import CoinbaseAuth
-from .public_client import PublicClient
-from .util import OrderSide, TimeInForce, SelfTradePrevention, Stop, OrderStatus, TransferType, ReportType, ReportFormat
+from zcoinbase import PublicClient, OrderSide, TimeInForce, SelfTradePrevention, Stop, OrderStatus, TransferType, \
+  ReportType, ReportFormat
 
 
 class AuthenticatedClient(PublicClient):
   def __init__(self, api_key, api_secret, passphrase,
-               rest_url='https://api.pro.coinbase.com'):
+               rest_url=PublicClient.PROD_URL):
     super().__init__(rest_url)
+    if not (api_key and api_secret and passphrase):
+      raise ValueError('api_key, api_secret, and passphrase are required.')
     self.auth = CoinbaseAuth(api_key, api_secret, passphrase)
 
+  @classmethod
+  def make_from_ini(cls, filename: Text):
+    """Makes an authenticated client from the given config file.
+
+    Example Config:
+      [Coinbase]
+      api_key: [YOUR API KEY]         (required)
+      api_secret: [YOUR API SECRET]   (required)
+      passphrase: [API PASSPHRASE]    (required)
+      backend: [PROD|SANDBOX|URL]     (optional, default PROD)
+
+    Args:
+      filename: The name of the file to read the config from.
+
+    Returns:
+      An initialized AuthenticatedClient from the given config file.
+    """
+    config = configparser.ConfigParser()
+    config.read(filename)
+    api_key = config['Coinbase']['api_key']
+    api_secret = config['Coinbase']['api_secret']
+    passphrase = config['Coinbase']['passphrase']
+    url = PublicClient.PROD_URL
+    if 'backend' in config['Coinbase']:
+      backend_config = config['Coinbase']['backend']
+      if backend_config == 'PROD':
+        url = PublicClient.PROD_URL
+      elif backend_config == 'SANDBOX':
+        url = PublicClient.SANDBOX_URL
+      else:
+        # If it's not PROD or SANDBOX, assume it's a URL.
+        url = backend_config
+    return cls(api_key=api_key, api_secret=api_secret, passphrase=passphrase, rest_url=url)
+
   def get_all_accounts(self):
     """List all accounts.
 
@@ -47,8 +85,7 @@ class AuthenticatedClient(PublicClient):
   def limit_order(self, side: OrderSide, product_id, price, size,
                   time_in_force=TimeInForce.GOOD_TILL_CANCEL,
                   cancel_after=None, post_only=None,
-                  self_trade_prevention=SelfTradePrevention.DECREASE_AND_CANCEL,
-                  stop=Stop.NONE, stop_price=None):
+                  self_trade_prevention=SelfTradePrevention.DECREASE_AND_CANCEL):
     """Places a limit order.
 
     https://docs.pro.coinbase.com/#place-a-new-order
@@ -60,13 +97,11 @@ class AuthenticatedClient(PublicClient):
     if post_only is not None and \
         (time_in_force is TimeInForce.IMMEDIATE_OR_CANCEL or time_in_force is TimeInForce.FILL_OR_KILL):
       raise ValueError('cannot set post_only with IMMEDIATE_OR_CANCEL or FILL_OR_KILL')
-    if stop is not Stop.NONE and stop_price is None:
-      raise ValueError('stop_price is required with LOSS or ENTRY')
 
     params = {
       'type': 'limit',
       'client_oid': AuthenticatedClient.make_order_uuid(),
-      'side': 'buy' if side == OrderSide.BUY else 'sell',
+      'side': side.value,
       'price': price,
       'size': size,
       'product_id': product_id,
@@ -77,28 +112,22 @@ class AuthenticatedClient(PublicClient):
       params['cancel_after'] = cancel_after
     if post_only is not None:
       params['post_only'] = post_only
-    if stop is not Stop.NONE:
-      params['stop'] = stop.value
-      params['stop_price'] = stop_price
     response = self._send_post('orders', params=params)
     response['client_oid'] = params['client_oid']
     return response
 
   def market_order(self, side: OrderSide, product_id, size=None, funds=None,
-                   self_trade_prevention=SelfTradePrevention.DECREASE_AND_CANCEL,
-                   stop=Stop.NONE, stop_price=None):
+                   self_trade_prevention=SelfTradePrevention.DECREASE_AND_CANCEL):
     """Places a market Order.
 
     https://docs.pro.coinbase.com/#place-a-new-order
     """
     if size is None and funds is None:
       raise ValueError('must specify size or funds for market order.')
-    if stop is not Stop.NONE and stop_price is None:
-      raise ValueError('stop_price is required with LOSS or ENTRY')
     params = {
       'type': 'market',
       'client_oid': AuthenticatedClient.make_order_uuid(),
-      'side': 'buy' if side == OrderSide.BUY else 'sell',
+      'side': side.value,
       'product_id': product_id,
       'stp': self_trade_prevention.value,
     }
@@ -106,9 +135,31 @@ class AuthenticatedClient(PublicClient):
       params['size'] = size
     if funds is not None:
       params['funds'] = funds
-    if stop is not Stop.NONE:
-      params['stop'] = stop.value
-      params['stop_price'] = stop_price
+    response = self._send_post('orders', params=params)
+    response['client_oid'] = params['client_oid']
+    return response
+
+  def stop_order(self, product_id,
+                 stop_type: Stop, stop_price,
+                 size=None, funds=None,
+                 self_trade_prevention=SelfTradePrevention.DECREASE_AND_CANCEL):
+    """Places a stop order with the given parameters."""
+    if stop_type is Stop.NONE:
+      raise ValueError('must specify stop_type as LOSS or ENTRY')
+    if not size and not funds:
+      raise ValueError('must specify size or funds.')
+    params = {
+      'side': 'sell' if stop_type is Stop.LOSS else 'buy',
+      'client_oid': AuthenticatedClient.make_order_uuid(),
+      'product_id': product_id,
+      'stp': self_trade_prevention.value,
+      'stop': stop_type.value,
+      'stop_price': stop_price,
+    }
+    if size:
+      params['size'] = size
+    if funds:
+      params['funds'] = funds
     response = self._send_post('orders', params=params)
     response['client_oid'] = params['client_oid']
     return response
